
/****************************************************************************
 Title  :   C Test program for the ADC FUNCTIONS library (test_adc.c)
 Author:    Chris efstathiou hendrix@otenet.gr
 Date:      6/Jun/2002
 Software:  AVR-GCC with AVR-AS
 Target:    any AVR device
 Comments:  This software is FREE.

*****************************************************************************/
/*
IMPORTANT:
It is my understanding that if you dont want to burn the AD Converter
never ever connect the ADC inputs to VCC. 
Also do not connect AREF or GND directly to the ADC inputs.
Use a resistor in series at least 10Kohm, otherwise you might burn the ADC as i did.
*/ 

#include <io.h>
#include "lcd_io.h"
#include "adc.h"

/* Function prototypes */
void draw_adc_display(void);
void display_adc_readings(void);
void wait(unsigned long msec);

const unsigned int adc_1Vx10=2046;

/*######################################################################################################*/
void wait(unsigned long msec)
{
 msec=msec*6000;
 while(msec) { msec--; }

return;
}

void draw_adc_display(void)
{
  
  lcd_gotoxy(0,0);
  lcd_puts_P("CH0       ");
  lcd_puts_P("CH4");
  lcd_gotoxy(0,1);
  lcd_puts_P("CH1       ");
  lcd_puts_P("CH5");
  lcd_gotoxy(0,2);
  lcd_puts_P("CH2       ");
  lcd_puts_P("CH6");
  lcd_gotoxy(0,3);
  lcd_puts_P("CH3       ");
  lcd_puts_P("CH7"); 

return;
}
/*######################################################################################################*/

void display_adc_readings(void)
{
unsigned long adc_reading=0;   
  
    
  adc_reading=adc_get_channel(0); 
  adc_reading=(adc_reading*10000)/adc_1Vx10;
  lcd_gotoxy(4,0); lcd_puti(adc_reading,3);
  
  
  adc_reading=adc_get_channel(4); 
  adc_reading=(adc_reading*10000)/adc_1Vx10;
  lcd_gotoxy(15,0); lcd_puti(adc_reading,3);
  
  
  adc_reading=adc_get_channel(1); 
  adc_reading=(adc_reading*10000)/adc_1Vx10;
  lcd_gotoxy(4,1); lcd_puti(adc_reading,3);

  
  adc_reading=adc_get_channel(5); 
  adc_reading=(adc_reading*10000)/adc_1Vx10;
  lcd_gotoxy(15,1); lcd_puti(adc_reading,3);
  
  
  adc_reading=adc_get_channel(2); 
  adc_reading=(adc_reading*10000)/adc_1Vx10;
  lcd_gotoxy(4,2); lcd_puti(adc_reading,3);
  
  
  adc_reading=adc_get_channel(6); 
  adc_reading=(adc_reading*10000)/adc_1Vx10;
  lcd_gotoxy(15,2); lcd_puti(adc_reading,3);
 
  
  adc_reading=adc_get_channel(3); 
  adc_reading=(adc_reading*10000)/adc_1Vx10;
  lcd_gotoxy(4,3); lcd_puti(adc_reading,3);
  
 
  adc_reading=adc_get_channel(7); 
  adc_reading=(adc_reading*10000)/adc_1Vx10;
  lcd_gotoxy(15,3); lcd_puti(adc_reading,3);

return;
}
/*######################################################################################################*/

void main(void)
{

  lcd_init();
  draw_adc_display(); 

  while(1)
      { 
         
         /* If you want adc measurements without the use of interrupt this is the way i prefer */
         /* 
         unsigned int adc_result=0;
         
         adc_init(ADC_INT_DIS, 200, 0); 
         adc_result=adc_convert(0);
         */
         
         /* Dont use ADC interrupt */
         adc_init(ADC_INT_DIS, 200, 0); 
         adc_convert(0);
         display_adc_readings();      
         wait(15);
         
         /* Scan channels 2,3,4 and 5 (SCAN MASK = 0X3C) */
         adc_init(ADC_INT_EN_SCAN, 200, (1<<ADC_CH2)|(1<<ADC_CH3)|(1<<ADC_CH4)|(1<<ADC_CH5)); 
         display_adc_readings();      
         wait(15);
                  
         adc_init(ADC_INT_EN_SLEEP, 200, 7); 
         display_adc_readings();
         wait(15);
 
         adc_init(ADC_INT_EN, 200, 6); 
         display_adc_readings();
         wait(15);
               
      }
 
}
/*######################################################################################################*/
/*                                         T H E   E N D                                                */
/*######################################################################################################*/


