/****************************************************************************
 Title  :   S file for the SOFTWARE UART FUNCTIONS library (softuart.s)
 Author:    Chris efstathiou hendrix@otenet.gr
 Date:      23/Oct/2002
 Software:  AVR-GCC with AVR-AS
 Target:    any AVR device
 Comments:  This software is FREE.


*****************************************************************************/

/*################### DO NOT CHANGE ANYTHING BELOW THIS LINE !!! #####################################*/
/*############## ALL CONFIGURATION IS DONE IN "softuart.cfg" file !!! ################################*/

#ifndef _IO_REG_MACRO_MODE_
#define _IO_REG_MACRO_MODE_  1     /* In case you have the new assignment mode io headers */
#endif

#ifndef  _SFR_ASM_COMPAT
#define  _SFR_ASM_COMPAT     1     /* This is for GCC 3.2 */
#endif  
/*
   this is for GCC 3.2 . It shouldn't be needed since i didn't define __ASSEMBLER__ anywhere
   but it does since __ASSEMBLER__ is defined somehow and i dont know why... 
*/
#if defined(__ASSEMBLER__)
#define _VECTOR(N) __vector_ ## N  
#endif




#include <io-avr.h>
#include "softuart.cfg"

/* CALCULATE THE I/O REGISTER ADDRESSES */
#if SREG > 0X3F 

#define S_TX_DDR_REG  SUART_TX_DDR_REG - 0X20
#define S_TX_PORT     SUART_TX_PORT    - 0X20
#define S_RX_DDR_REG  SUART_RX_DDR_REG - 0X20
#define S_RX_PIN_REG  SUART_RX_PIN_REG - 0X20
#define S_INT_REG     SUART_INT_REG    - 0X20
#define AVR_TCNT      SUART_TCNT       - 0X20
#define AVR_TCCR      SUART_TCCR       - 0X20
#define AVR_SREG      SREG             - 0X20
#define AVR_EIFR      SUART_INT_FLAG_REG - 0X20

#elif SREG <= 0X3F  

#define S_TX_DDR_REG  SUART_TX_DDR_REG 
#define S_TX_PORT     SUART_TX_PORT    
#define S_RX_DDR_REG  SUART_RX_DDR_REG 
#define S_RX_PIN_REG  SUART_RX_PIN_REG 
#define S_INT_REG     SUART_INT_REG 
#define AVR_TCNT      SUART_TCNT 
#define AVR_TCCR      SUART_TCCR       
#define AVR_SREG      SREG 
#define AVR_EIFR      SUART_INT_FLAG_REG 

#endif  

/* ERROR CHECKING AND MESSAGES */
#if   SUART_WILL_USE_A_TIMER == 0 && SUART_DYNAMIC_BAUD_CHANGE == 0

#if    SUART_DELAY1 <=0  
#error " BAUDRATE TOO HIGH "
#elif  SUART_DELAY1 > 255 
#error " BAUDRATE TOO LOW "
#endif

#if    SUART_DELAY2 <= 2  
#error " BAUDRATE TOO HIGH "
#elif  SUART_DELAY2 > 255 
#error " BAUDRATE TOO LOW "
#endif

#if    SUART_DELAY3 <= 2  
#error " BAUDRATE TOO HIGH "
#elif  SUART_DELAY3 > 255 
#error " BAUDRATE TOO LOW "
#endif

#elif SUART_WILL_USE_A_TIMER == 1  && SUART_DYNAMIC_BAUD_CHANGE == 0

#if   TIMER0_VALUE2 <= 0 
#warning " BAUDRATE TOO HIGH "
#endif

#if TIMER0_VALUE1 < 0 || SUART_BAUDRATE_DELAY < (SUART_CORRECTION_FACTOR+10)
#error   " BAUDRATE TOO HIGH "
#endif

#endif



/* REGISTER DEFINITIONS */
#define          z_low        r30          
#define          z_high       r31          
#define          s_index      r27
#define          s_temp2      r27          
#define          s_temp1      r26 
#define          s_sreg       r25
#define          s_Rxbyte     r24 
#define          s_Txbyte     r24          
#define          s_bitcnt     r23


/*#######################################################################################################*/

                 .global SUART_PUTCHAR
                 .func SUART_PUTCHAR

SUART_PUTCHAR:   
              
                 ldi      s_bitcnt,9+SUART_STOP_BITS
                 com      s_Txbyte                  /* INVERT THE BYTE TO BE TRANSMITED */
                 sec                                /* SET CARRY (CARRY=1) */

#if    SUART_USE_RS232_LOGIC == 0 
suart_putchar0:  brcc     suart_putchar1            /* IF CARRY IS CLEAR SET THE TX PIN TO 1 */
#elif  SUART_USE_RS232_LOGIC == 1
suart_putchar0:  brcs     suart_putchar1            /* IF CARRY IS SET SET THE TX PIN TO 1 */
#endif 

#if SUART_DYNAMIC_IO_SETTINGS == 1

                 clr      z_high                    /* cbi using variables */
                 lds      z_low,  suart_port
                 ld       s_temp1, z
                 lds      s_temp2, suart_tx_pin         
                 com      s_temp2 
                 and      s_temp1, s_temp2
                 st       z, s_temp1                     

                 rjmp     suart_putchar2          
                                                    /* sbi using variables */
suart_putchar1:  clr      z_high
                 lds      z_low,  suart_port
                 ld       s_temp1, z
                 lds      s_temp2, suart_tx_pin 
                 or       s_temp1, s_temp2
                 st       z, s_temp1           
                 nop

#elif SUART_DYNAMIC_IO_SETTINGS == 0
                 cbi      S_TX_PORT,SUART_TX_PIN    /* ELSE SET THE TX PIN TO 1 */
                 rjmp     suart_putchar2          

suart_putchar1:  sbi      S_TX_PORT,SUART_TX_PIN         
                 nop
#endif

#if SUART_WILL_USE_A_TIMER == 0  &&  SUART_DYNAMIC_BAUD_CHANGE == 1                

suart_putchar2:  lds      s_temp1, suart_delay2     /* 1/2 bit delay loop */
                 subi     s_temp1, 1                /* SUBTRACT 1 TO COMPENSATE FOR THE Lds's 2 CYCLES */ 
                 nop                                /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
suart_tx_delay1: dec      s_temp1
                 brne     suart_tx_delay1
                 
                 lds      s_temp1, suart_delay3     /* 1/2 bit delay loop. total delay 1 bit*/
                 subi     s_temp1, 1                /* SUBTRACT 1 TO COMPENSATE FOR THE Lds's 2 CYCLES */  
                 nop                                /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
suart_tx_delay2: dec      s_temp1
                 brne     suart_tx_delay2

#elif SUART_WILL_USE_A_TIMER == 1  &&  SUART_DYNAMIC_BAUD_CHANGE == 1 

suart_putchar2:  clr      s_temp1                   /* 1 bit delay loop */
                 out      AVR_TCNT, s_temp1
                                                  
                 lds      s_temp2, timer_value1
                                 
suart_tx_delay1: in       s_temp1, AVR_TCNT
                 cp       s_temp1, s_temp2
                 brlo     suart_tx_delay1

#elif SUART_WILL_USE_A_TIMER == 0  &&  SUART_DYNAMIC_BAUD_CHANGE == 0 

suart_putchar2:  ldi      s_temp1, SUART_DELAY2     /* 1/2 bit delay loop */       
suart_tx_delay1: dec      s_temp1                   /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */
                 brne     suart_tx_delay1

                 ldi      s_temp1, SUART_DELAY3     /* 1/2 bit delay loop */ 
suart_tx_delay2: dec      s_temp1                   /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */
                 brne     suart_tx_delay2           

#elif SUART_WILL_USE_A_TIMER == 1  &&  SUART_DYNAMIC_BAUD_CHANGE == 0 

suart_putchar2:  clr      s_temp1                   /* 1 bit delay loop using a timer */
                 out      AVR_TCNT, s_temp1
                                                  
                 ldi      s_temp2, TIMER0_VALUE1 
                                 
suart_tx_delay1: in       s_temp1, AVR_TCNT
                 cp       s_temp1, s_temp2
                 brlo     suart_tx_delay1

#endif

                 lsr      s_Txbyte                  /* SHIFT LEFT TX BYTE THROUGH CARRY */
                 dec      s_bitcnt                  /* DECREMENT BITCOUNT */
                 brne     suart_putchar0            /* TRANSMIT NEXT BIT  */
                                     
                 ret                             


.endfunc
                
/*####################################################################################################*/
/*####################################################################################################*/
#if SUART_DYNAMIC_IO_SETTINGS == 0
/*####################################################################################################*/
/*####################################################################################################*/

                 .global SUART_RX_FUNCTION
                 .func   SUART_RX_FUNCTION

SUART_RX_FUNCTION: 


 
/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/
#if   SUART_WILL_USE_INTERRUPT  == 1   /* INTERRUPT MODE FOR NON DYNAMIC PORT-PIN RECEIVING */
/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/

/*#############    It took at least 5 cycles to get here ###############*/

                 push    s_sreg                     /* SAVE THE SREG REGISTER FIRST */
                 in      s_sreg, AVR_SREG

#if SUART_WILL_USE_A_TIMER == 0  &&  SUART_DYNAMIC_BAUD_CHANGE == 1                    

                 push    s_temp1                    /* SAVE THE REGISTERS TO BE USED */
                 push    s_temp2
                 push    z_low
                 push    z_high
                 push    s_Rxbyte                    
                 push    s_bitcnt 

                 ldi     s_bitcnt, 8                /* STOP BIT IS EATEN AND USED FOR RX DATA PROCESSING */
              
suart_getchar1:  lds     s_temp1, suart_delay1      /* 1/2 bit delay loop to be counted only once */
suart_rx_delay1: dec     s_temp1                     
                 brne    suart_rx_delay1                    
                                                
suart_getchar2:  lds     s_temp1, suart_delay2      /* 1/2 bit delay loop */
                 subi    s_temp1, 1                 /* SUBTRACT 1 TO COMPENSATE FOR THE Lds's 2 CYCLES */   
                 nop                                /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */   
suart_rx_delay2: dec     s_temp1                     
                 brne    suart_rx_delay2
                 
                 lds     s_temp1, suart_delay3      /* 1/2 bit delay loop */
                 subi    s_temp1, 1                 /* SUBTRACT 1 TO COMPENSATE FOR THE Lds's 2 CYCLES */  
                 nop                                /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */   
suart_rx_delay3: dec     s_temp1                     
                 brne    suart_rx_delay3                           
 
                                                        
#elif SUART_WILL_USE_A_TIMER == 1  &&  SUART_DYNAMIC_BAUD_CHANGE == 1  
                 push    s_temp1                     
                 clr     s_temp1                    /* 1/2 bit delay loop to be counted only once */
                 out     AVR_TCNT, s_temp1
/* Dead time start(TIMER IS ALREADY COUNTING). */ 
                 push    s_temp2                   
                 push    z_low                     
                 push    z_high                 
                 push    s_Rxbyte                    
                 push    s_bitcnt 
                 ldi     s_bitcnt, 8                /* STOP BIT IS EATEN AND USED FOR RX DATA PROCESSING */              
                 lds     s_temp2, timer_value2    
/***************   Dead time end   ************/
suart_rx_delay1: in      s_temp1, AVR_TCNT
                 cp      s_temp1, s_temp2
                 brlo    suart_rx_delay1

suart_getchar2:  clr      s_temp1                   /* 1 bit delay loop using a timer */
                 out      AVR_TCNT, s_temp1
/* Dead time start(TIMER IS ALREADY COUNTING). */ 
                 lds      s_temp2, timer_value1
/***************   Dead time end   ************/
suart_rx_delay2: in       s_temp1, AVR_TCNT
                 cp       s_temp1, s_temp2
                 brlo     suart_rx_delay2
                 
#elif SUART_WILL_USE_A_TIMER == 0  &&  SUART_DYNAMIC_BAUD_CHANGE == 0

                 push    s_temp1 
                 push    s_temp2                       
                 push    z_low                     
                 push    z_high                    
                 push    s_Rxbyte                    
                 push    s_bitcnt 

                 ldi     s_bitcnt, 8                /* STOP BIT IS EATEN AND USED FOR RX DATA PROCESSING */
               
suart_getchar1:  ldi     s_temp1, SUART_DELAY1      /* 1/2 bit delay loop to be counted only once */
suart_rx_delay1: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
                 brne    suart_rx_delay1                    
                                                
suart_getchar2:  ldi     s_temp1, SUART_DELAY2      /* 1/2 bit delay loop */
suart_rx_delay2: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
                 brne    suart_rx_delay2
     
                 ldi     s_temp1, SUART_DELAY3      /* 1/2 bit delay loop */   
suart_rx_delay3: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
                 brne    suart_rx_delay3                                 
                                                        
#elif SUART_WILL_USE_A_TIMER == 1  &&  SUART_DYNAMIC_BAUD_CHANGE == 0

                 push    s_temp1                               
                 clr     s_temp1                    /* 1/2 bit delay loop to be counted only once */
                 out     AVR_TCNT, s_temp1
/* Dead time start(TIMER IS ALREADY COUNTING). */
                 push    s_temp2
                 push    z_low
                 push    z_high
                 push    s_Rxbyte
                 push    s_bitcnt 

                 ldi     s_bitcnt, 8                /* STOP BIT IS EATEN AND USED FOR RX DATA PROCESSING */   
                 ldi     s_temp2, TIMER0_VALUE2

/***************   Dead time end   ************/
suart_rx_delay1: in      s_temp1, AVR_TCNT
                 cp      s_temp1, s_temp2
                 brlo    suart_rx_delay1
            
suart_getchar2:  clr      s_temp1                   /* 1 bit delay loop using a timer */
                 out      AVR_TCNT, s_temp1
/* Dead time start(TIMER IS ALREADY COUNTING). */ 
                 ldi      s_temp2, TIMER0_VALUE1 
/***************   Dead time end   ************/                              
suart_rx_delay2: in       s_temp1, AVR_TCNT
                 cp       s_temp1, s_temp2
                 brlo     suart_rx_delay2
                                                
#endif
             
                 
/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/
#elif SUART_WILL_USE_INTERRUPT  == 0  /* NON INTERRUPT MODE FOR NON DYNAMIC PORT-PIN RECEIVING */
/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/

#if    SUART_USE_RS232_LOGIC == 0  
suart_wait_high: sbis    S_RX_PIN_REG, SUART_RX_PIN /* WAIT FOR A HIGH TO LOW TRANSITION OF THE RX PIN */
                 rjmp    suart_wait_high            
suart_wait_low:  sbic    S_RX_PIN_REG, SUART_RX_PIN          
                 rjmp    suart_wait_low            
#elif  SUART_USE_RS232_LOGIC == 1
suart_wait_low:  sbic    S_RX_PIN_REG, SUART_RX_PIN /* WAIT FOR A LOW TO HIGH TRANSITION OF THE RX PIN */
                 rjmp    suart_wait_low            
suart_wait_high: sbis    S_RX_PIN_REG, SUART_RX_PIN          
                 rjmp    suart_wait_high            

#endif
 
                 ldi     s_bitcnt, 8                /* STOP BIT IS EATEN AND USED FOR RX DATA PROCESSING */                                                                                          

#if SUART_WILL_USE_A_TIMER == 0  &&  SUART_DYNAMIC_BAUD_CHANGE == 1                    
              
                 lds     s_temp1, suart_delay1      /* 1/2 bit delay loop to be counted only once */
suart_rx_delay1: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
                 brne    suart_rx_delay1                    
                                                
                
suart_getchar2:  lds     s_temp1, suart_delay2      /* 1 bit delay loop */
                 subi    s_temp1, 1                 /* SUBTRACT 1 TO COMPENSATE FOR THE Lds's 2 CYCLES */  
                 nop                                /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */   
suart_rx_delay2: dec     s_temp1                     
                 brne    suart_rx_delay2                       

                 lds     s_temp1, suart_delay3      /* 1/2 bit delay loop */
                 subi    s_temp1, 1                 /* SUBTRACT 1 TO COMPENSATE FOR THE Lds's 2 CYCLES */  
                 nop                                /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */   
suart_rx_delay3: dec     s_temp1                     
                 brne    suart_rx_delay3                 

#elif SUART_WILL_USE_A_TIMER == 1  &&  SUART_DYNAMIC_BAUD_CHANGE == 1
                                         
                 clr     s_temp2                    /* 1/2 bit delay loop to be counted only once */
                 out     AVR_TCNT, s_temp2
/* Dead time start(TIMER IS ALREADY COUNTING). */ 
                 lds     s_temp2, timer_value2    
/***************   Dead time end   ************/
suart_rx_delay1: in      s_temp1, AVR_TCNT
                 cp      s_temp1, s_temp2
                 brlo    suart_rx_delay1

suart_getchar2:  clr     s_temp1                    /* 1 bit delay loop using a timer */
                 out     AVR_TCNT, s_temp1
/* Dead time start(TIMER IS ALREADY COUNTING). */ 
                 lds     s_temp2, timer_value1
/***************   Dead time end   ************/ 
suart_rx_delay2: in      s_temp1, AVR_TCNT
                 cp      s_temp1, s_temp2
                 brlo    suart_rx_delay2

#elif SUART_WILL_USE_A_TIMER == 0  &&  SUART_DYNAMIC_BAUD_CHANGE == 0

                 ldi     s_temp1, SUART_DELAY1      /* 1/2 bit delay loop to be counted only once */
suart_rx_delay1: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */
                 brne    suart_rx_delay1                    
                                                
suart_getchar2:  ldi     s_temp1, SUART_DELAY2      /* 1/2 bit delay loop */
suart_rx_delay2: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
                 brne    suart_rx_delay2
              
                 ldi     s_temp1, SUART_DELAY3      /* 1/2 bit delay loop */   
suart_rx_delay3: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
                 brne    suart_rx_delay3                       

#elif SUART_WILL_USE_A_TIMER == 1  &&  SUART_DYNAMIC_BAUD_CHANGE == 0

                 clr     s_temp2                    /* 1/2 bit delay loop to be counted only once */
                 out     AVR_TCNT, s_temp2
/* Dead time start(TIMER IS ALREADY COUNTING). */                        
                 ldi     s_temp2, TIMER0_VALUE2     
/***************   Dead time end   ************/                                                       
suart_rx_delay1: in      s_temp1, AVR_TCNT
                 cp      s_temp1, s_temp2
                 brlo    suart_rx_delay1
            
suart_getchar2:  clr     s_temp1                    /* 1 bit delay loop using a timer */
                 out     AVR_TCNT, s_temp1
/* Dead time start(TIMER IS ALREADY COUNTING). */ 
                 ldi     s_temp2, TIMER0_VALUE1 
/***************   Dead time end   ************/                               
suart_rx_delay2: in      s_temp1, AVR_TCNT
                 cp      s_temp1, s_temp2
                 brlo    suart_rx_delay2

#endif

       

/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/
#endif
/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/                 

                 clc                               /* CLEAR CARRY */                

#if    SUART_USE_RS232_LOGIC == 0                  
                 sbic    S_RX_PIN_REG, SUART_RX_PIN /* IF RX PIN IS LOW CLEAR CARRY */             
#elif  SUART_USE_RS232_LOGIC == 1
                 sbis    S_RX_PIN_REG, SUART_RX_PIN /* IF RX PIN IS HIGH CLEAR CARRY */             
#endif


                 sec
suart_rotate:    ror     s_Rxbyte                  /* ROTATE RIGHT THE RX BYTE THROUGH CARRY */      
                 dec     s_bitcnt                  /* DECREMENT BITCOUNT */
                 breq    suart_wr_buf_0            /* IF THIS WAS THE LAST BIT WRITE BYTE TO BUFFER */
                                                   /* ELSE */                                               
                 rjmp    suart_getchar2            /* GO GET NEXT BIT */ 

/* ######### RECEIVING IS COMPLETE SO LETS START WRITTING THE BUFFER ############## */
/* ######### RX DATA PROCESSING FOR NON DYNAMIC PORT-PIN MODE        ############## */
suart_wr_buf_0:  sts     rx_c, r24                           
                 lds     s_index, suart_rx_count

  

#if SUART_WILL_USE_INTERRUPT  == 1 && SUART_RECEIVE_STRINGS_ONLY == 0
                             
                 lds     s_temp1, suart_receive_string
                 cpi     s_temp1, 1
                 brne    suart_wr_buf_1             
#endif                 

                 cpi     s_Rxbyte, RX_TERMINATION_CHAR          
                 breq    suart_term_recv0     

#if  SUART_RECEIVE_STRINGS_ONLY == 0                  

suart_wr_buf_1:  cpi     s_index, SUART_BUF_SIZE     
                 breq    suart_buf_reset                         

#endif              

suart_wr_buf_2:  ldi     z_low, lo8(suart_rx_buffer)
                 ldi     s_temp1, hi8(suart_rx_buffer)         
                 add     z_low, s_index              
                 adc     z_high,s_temp1                       
suart_wr_buf_3:  st      Z+, s_Rxbyte                          
                 inc     s_index                      
                 clr     s_temp1                              
                 st      Z, s_temp1                            
                 
/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/
#if SUART_WILL_USE_INTERRUPT  == 1
/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/

#if  SUART_RECEIVE_STRINGS_ONLY == 0
exit_suart_0:    lds     s_temp1, suart_set_mark
                 tst     s_temp1
                 dec     s_temp1
                 breq    suart_term_recv1
                 sts     suart_set_mark, s_temp1
#endif


exit_suart_1:    sts     suart_rx_count, s_index
#if         EXT_INT_EDGE_TRIGERING == 1 
                 ldi     s_temp1, (1<<SUART_INT_FLAG)
                 out     AVR_EIFR, s_temp1
#endif
                 pop     s_bitcnt
                 pop     s_Rxbyte                               
                 pop     z_high      
                 pop     z_low
                 pop     s_temp2
                 pop     s_temp1
                 out     AVR_SREG, s_sreg
                 pop     s_sreg

#if    SUART_USE_RS232_LOGIC == 0 
exit_suart_2:    sbis    S_RX_PIN_REG, SUART_RX_PIN
                 rjmp    exit_suart_2                        
#elif  SUART_USE_RS232_LOGIC == 1 
exit_suart_2:    sbic    S_RX_PIN_REG, SUART_RX_PIN
                 rjmp    exit_suart_2         
#endif
                 reti                                        

suart_term_recv0:inc     s_index
suart_term_recv1:in      s_temp1, S_INT_REG                       
                 andi    s_temp1, (~(1<<SUART_INT_BIT))                   
                 out     S_INT_REG, s_temp1                     
#if  SUART_RECEIVE_STRINGS_ONLY == 0
                 clr     s_temp1
                 sts     suart_set_mark, s_temp1
#endif
                 sts     suart_string_received, s_index  
                 rjmp    exit_suart_1

#if  SUART_RECEIVE_STRINGS_ONLY == 0

suart_buf_reset: ldi     s_index, SUART_BUF_SIZE+2
                 sts     suart_set_mark, s_index
                 clr     s_index                     
                 ldi     z_low,  lo8(suart_rx_buffer)        
                 ldi     z_high, hi8(suart_rx_buffer)        
                 rjmp    suart_wr_buf_3 

#endif

/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/
#elif SUART_WILL_USE_INTERRUPT  == 0
/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/

exit_suart_0:    sts     suart_rx_count, s_index 
#if  SUART_RECEIVE_STRINGS_ONLY == 0
                 lds     s_temp1, suart_set_mark
                 tst     s_temp1
                 breq    suart_term_recv0
                 dec     s_temp1
                 sts     suart_set_mark, s_temp1
#endif
exit_suart_1:    ret                                         

suart_term_recv0:inc     s_index
                 sts     suart_string_received, s_index           
                 sts     suart_rx_count, s_index
#if  SUART_RECEIVE_STRINGS_ONLY == 0
                 clr     s_temp1                     
                 sts     suart_set_mark, s_temp1
#endif
                 ret

#if  SUART_RECEIVE_STRINGS_ONLY == 0
suart_buf_reset: ldi     s_index, SUART_BUF_SIZE+2
                 sts     suart_set_mark, s_index
                 clr     s_index                     
                 ldi     z_low,  lo8(suart_rx_buffer)        
                 ldi     z_high, hi8(suart_rx_buffer)        
                 rjmp    suart_wr_buf_3 
#endif
/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/
#endif
/*1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111*/

.endfunc

/*####################################################################################################*/
/*####################################################################################################*/
#elif SUART_DYNAMIC_IO_SETTINGS == 1   /* NON INTERRUPT DYNAMIC PORT-PIN MODE */
/*####################################################################################################*/
/*####################################################################################################*/

                 .global SUART_RX_FUNCTION
                 .func   SUART_RX_FUNCTION

SUART_RX_FUNCTION: 

#if    SUART_USE_RS232_LOGIC == 0  

                 clr      z_high
                 lds      z_low,  suart_port
                 subi     z_low, 2
                 lds      s_temp2, suart_rx_pin  
suart_wait_high: ld       s_temp1, z
                 and      s_temp1, s_temp2
                 tst      s_temp1                
                 breq     suart_wait_high            

suart_wait_low:  ld       s_temp1, z
                 and      s_temp1, s_temp2
                 tst      s_temp1                
                 brne     suart_wait_low 

#elif  SUART_USE_RS232_LOGIC == 1
                 clr      z_high
                 lds      z_low,  suart_port
                 subi     z_low, 2
                 lds      s_temp2, suart_rx_pin 
suart_wait_low:  ld       s_temp1, z
                 and      s_temp1, s_temp2
                 tst      s_temp1                
                 brne     suart_wait_low 

suart_wait_high: ld       s_temp1, z
                 and      s_temp1, s_temp2
                 tst      s_temp1                
                 breq     suart_wait_high                
#endif
 
                 ldi     s_bitcnt, 8                /* STOP BIT IS EATEN AND USED FOR RX DATA PROCESSING */                                                                                          

#if SUART_WILL_USE_A_TIMER == 0  &&  SUART_DYNAMIC_BAUD_CHANGE == 1                    
              
                 lds     s_temp1, suart_delay1      /* 1/2 bit delay loop to be counted only once */
suart_rx_delay1: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
                 brne    suart_rx_delay1                    
                                                
                
suart_getchar2:  lds     s_temp1, suart_delay2      /* 1 bit delay loop */
                 subi    s_temp1, 1                 /* SUBTRACT 1 TO COMPENSATE FOR THE Lds's 2 CYCLES */  
                 nop                                /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */   
suart_rx_delay2: dec     s_temp1                     
                 brne    suart_rx_delay2                       

                 lds     s_temp1, suart_delay3      /* 1/2 bit delay loop */
                 subi    s_temp1, 1                 /* SUBTRACT 1 TO COMPENSATE FOR THE Lds's 2 CYCLES */  
                 nop                                /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */   
suart_rx_delay3: dec     s_temp1                     
                 brne    suart_rx_delay3                 

#elif SUART_WILL_USE_A_TIMER == 1  &&  SUART_DYNAMIC_BAUD_CHANGE == 1
                                         
                 clr     s_temp2                    /* 1/2 bit delay loop to be counted only once */
                 out     AVR_TCNT, s_temp2
/* Dead time start(TIMER IS ALREADY COUNTING). */ 
                 lds     s_temp2, timer_value2    
/***************   Dead time end   ************/
suart_rx_delay1: in      s_temp1, AVR_TCNT
                 cp      s_temp1, s_temp2
                 brlo    suart_rx_delay1

suart_getchar2:  clr     s_temp1                    /* 1 bit delay loop using a timer */
                 out     AVR_TCNT, s_temp1
/* Dead time start(TIMER IS ALREADY COUNTING). */ 
                 lds     s_temp2, timer_value1
/***************   Dead time end   ************/ 
suart_rx_delay2: in      s_temp1, AVR_TCNT
                 cp      s_temp1, s_temp2
                 brlo    suart_rx_delay2

#elif SUART_WILL_USE_A_TIMER == 0  &&  SUART_DYNAMIC_BAUD_CHANGE == 0

                 ldi     s_temp1, SUART_DELAY1      /* 1/2 bit delay loop to be counted only once */
suart_rx_delay1: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */
                 brne    suart_rx_delay1                    
                                                
suart_getchar2:  ldi     s_temp1, SUART_DELAY2      /* 1/2 bit delay loop */
suart_rx_delay2: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
                 brne    suart_rx_delay2
              
                 ldi     s_temp1, SUART_DELAY3      /* 1/2 bit delay loop */   
suart_rx_delay3: dec     s_temp1                    /* TOTAL DELAY = (suart_delayX*3)*(1/F_CPU) */ 
                 brne    suart_rx_delay3                       

#elif SUART_WILL_USE_A_TIMER == 1  &&  SUART_DYNAMIC_BAUD_CHANGE == 0

                 clr     s_temp2                    /* 1/2 bit delay loop to be counted only once */
                 out     AVR_TCNT, s_temp2
/* Dead time start(TIMER IS ALREADY COUNTING). */                        
                 ldi     s_temp2, TIMER0_VALUE2     
/***************   Dead time end   ************/                                                       
suart_rx_delay1: in      s_temp1, AVR_TCNT
                 cp      s_temp1, s_temp2
                 brlo    suart_rx_delay1
            
suart_getchar2:  clr     s_temp1                    /* 1 bit delay loop using a timer */
                 out     AVR_TCNT, s_temp1
/* Dead time start(TIMER IS ALREADY COUNTING). */ 
                 ldi     s_temp2, TIMER0_VALUE1 
/***************   Dead time end   ************/                               
suart_rx_delay2: in      s_temp1, AVR_TCNT
                 cp      s_temp1, s_temp2
                 brlo    suart_rx_delay2

#endif

                 clc                               /* CLEAR CARRY */                

#if    SUART_USE_RS232_LOGIC == 0                  
                 nop
                 clr      z_high                   /* skip if bit is clear using variables */
                 lds      z_low,  suart_port       /* if bit is clear leave carry cleared  */
                 subi     z_low, 2                 /* Subtract 2 to get the pin register address */
                 ld       s_temp1, z                                      
                 lds      s_temp2, suart_rx_pin         
                 and      s_temp1, s_temp2
                 tst      s_temp1                
                 breq     suart_rotate
        
#elif  SUART_USE_RS232_LOGIC == 1
                 nop
                 clr      z_high                   /* skip if bit is set using variables */
                 lds      z_low,  suart_port       /* if bit is clear leave carry cleared  */
                 subi     z_low, 2                 /* Subtract 2 to get the pin register address */
                 ld       s_temp1, z
                 lds      s_temp2, suart_rx_pin         
                 and      s_temp1, s_temp2
                 tst      s_temp1                
                 brne     suart_rotate            /* IF RX PIN IS HIGH CLEAR CARRY */             
#endif

                 sec                               /* SET CARRY IF NOT SKIPPED BY SBI-CBI ABOVE */
suart_rotate:    ror     s_Rxbyte                  /* ROTATE RIGHT THE RX BYTE THROUGH CARRY */      
                 dec     s_bitcnt                  /* DECREMENT BITCOUNT */
                 breq    suart_wr_buf_0            /* IF THIS WAS THE LAST BIT WRITE BYTE TO BUFFER */
                                                   /* ELSE */                                               
                 rjmp    suart_getchar2            /* GO GET NEXT BIT */ 

/* ######### RECEIVING IS COMPLETE SO LETS START WRITTING THE BUFFER ############## */
/* ######### RX DATA PROCESSING FOR DYNAMIC PORT-PIN MODE            ############## */
suart_wr_buf_0:  sts     rx_c, r24                           
                 lds     s_index, suart_rx_count

                 cpi     s_Rxbyte, RX_TERMINATION_CHAR          
                 breq    suart_term_recv0     

#if  SUART_RECEIVE_STRINGS_ONLY == 0                  

suart_wr_buf_1:  cpi     s_index, SUART_BUF_SIZE     
                 breq    suart_buf_reset                         

#endif              

suart_wr_buf_2:  ldi     z_low, lo8(suart_rx_buffer)
                 ldi     s_temp1, hi8(suart_rx_buffer)         
                 add     z_low, s_index              
                 adc     z_high,s_temp1                       
suart_wr_buf_3:  st      Z+, s_Rxbyte                          
                 inc     s_index                      
                 clr     s_temp1                              
                 st      Z, s_temp1                            
                 
exit_suart_0:    sts     suart_rx_count, s_index 
#if  SUART_RECEIVE_STRINGS_ONLY == 0
                 lds     s_temp1, suart_set_mark
                 tst     s_temp1
                 breq    suart_term_recv0
                 dec     s_temp1
                 sts     suart_set_mark, s_temp1
#endif
exit_suart_1:    ret                                         

suart_term_recv0:inc     s_index
                 sts     suart_string_received, s_index           
                 sts     suart_rx_count, s_index
#if  SUART_RECEIVE_STRINGS_ONLY == 0
                 clr     s_temp1                     
                 sts     suart_set_mark, s_temp1
#endif
                 ret

#if  SUART_RECEIVE_STRINGS_ONLY == 0
suart_buf_reset: ldi     s_index, SUART_BUF_SIZE+2
                 sts     suart_set_mark, s_index
                 clr     s_index                     
                 ldi     z_low,  lo8(suart_rx_buffer)        
                 ldi     z_high, hi8(suart_rx_buffer)        
                 rjmp    suart_wr_buf_3 
#endif


.endfunc

/*####################################################################################################*/
/*####################################################################################################*/
#endif   /* #elif SUART_DYNAMIC_IO_SETTINGS == 1  */
/*####################################################################################################*/
/*####################################################################################################*/



                                      /* THANK GOD IS OVER!!! */
/*########################################################################################################*/
/*                                     T   H   E      E   N   D                                           */
/*########################################################################################################*/                

