/****************************************************************************
 Title  :   C Test program for the Software UART FUNCTIONS library (test_uart.c)
 Author:    Chris efstathiou hendrix@otenet.gr
 Date:      12/Oct/2002
 Software:  AVR-GCC with AVR-AS
 Target:    any AVR device
 Comments:  Based on application note AVR 305 
            This software is FREE.

*****************************************************************************/
/*
   If you use an external interrupt "suart_command(SUART_GET_STRING)" and "suart_command(SUART_RX_OFF)"    
   are functional but if you do not use an external interrupt they perform dummy operations           
   because they are not really needed since you initiate receiving.                                   
   When an external interrupt is used there is no need to wait for the string to arrive but when      
   you do not use an external interrupt the receiving is actually a function thus you need to wait    
   In any case "suart_gets()" returns a pointer to suart_rx_buffer if data arrived                    
   or a NULL pointer if nothing was received or receiving is not complete yet, so if you use          
   an external interrupt check occasionally to see if the returned pointer is 0 and if it is not      
   then valid data were received and the returned pointer points to suart_rx_buffer.   
*/     

 
#include <io.h>
#include "suart.h"




void main(void)
{

unsigned char buffer[20];
unsigned char string[]="\r\nSoftware UART testing...";
static unsigned char eeprom_string[]__attribute__((section(".eeprom")))="suart_puts_e Testing...";


/* Initialize the software uart to 115200 bps and enable receiving. */
   
#if SUART_DYNAMIC_BAUD_CHANGE == 1  

unsigned long baudrate=115200;
unsigned char error=0;

/* If there is an error go to a lower BAUDRATE */
while( (error=suart_init(baudrate)) == SUART_BAUDRATE_TOO_HIGH )
    {
      baudrate/=2;     
      if(baudrate==28800) { baudrate=38400; }
    } 

#elif SUART_DYNAMIC_BAUD_CHANGE == 0

suart_init();

#endif

#if SUART_DYNAMIC_IO_SETTINGS == 1
suart_set_io(PORTD,4,5);
#endif

while(1)
    {
 
#if SUART_WILL_USE_INTERRUPT == 1 &&  SUART_RECEIVE_STRINGS_ONLY == 0
#warning " SUART IN INTERRUPT MODE "
      suart_puts(string);
      suart_puts_P("\r\n");
      /* ENABLE STRING RECEIVING */
      suart_puts_P("Type something + ENTER\r\n");
      suart_gets();                                  /* ENABLE STRING RECEIVING */
      while(suart_string_received == 0);
      suart_puts_P("sizeof(received_string)= ");
      suart_puti(suart_string_received, 0);
      
/*
      "suart_get_buffer(buffer);" gets the whole buffer. After the buffer has been read the used chars
      are removed from the suart_rx_buffer and their space is reclaimed.
      You can also get the actual suart_rx_buffer as demonstrated below. This way the suart_rx_buffer is
      left intact. 
*/
      suart_puts_P("\r\n1) ACTUAL BUFFER CONTENTS: ");
      suart_puts((unsigned char*)suart_rx_buffer);
      suart_puts_P("\r\n2) RETRIEVED BUFFER DATA: ");
      suart_get_buf(buffer, suart_string_received);
      suart_puts(buffer);
      suart_puts_P("\r\n");
      
      suart_puts_P("Type at least 5 chars\r\n");
      suart_set_mark=5;                             /* set the max wanted received chars */
      suart_command(SUART_RX_ON);                   /* After 5 chars receiving ends automatically */
      while(suart_set_mark);
      suart_get_buf(buffer, suart_rx_count);
      suart_puts(buffer);                           /* After the buffer has been read the chars are */
      suart_puts_P("\r\n");                         /* no longer in the buffer and more space is usable */
      
      suart_puts_P("Type 5 more chars \r\n");
      suart_command(SUART_RX_ON);                   /* ENABLE RAW DATA RECEIVING. (All chars are accepted) */
      while(suart_rx_count < 5 );                   /* receiving is not stoping automatically */
      suart_get_buf(buffer, suart_rx_count);
      suart_puts(buffer);                           /* After the buffer has been read the chars are */
      suart_puts_P("\r\n");                         /* no longer in the buffer and more space is usable */
      suart_command(SUART_RX_OFF);

#elif SUART_WILL_USE_INTERRUPT == 0  &&  SUART_RECEIVE_STRINGS_ONLY == 0
#warning " SUART IN NON INTERRUPT MODE "
      suart_puts(string);
      suart_puts_P("\r\n");
      /* ENABLE STRING RECEIVING */
      suart_puts_P("Type something\r\n");
      suart_gets();
      suart_puts_P("sizeof(received_string)= ");
      suart_puti(suart_string_received, 0);
/*
      "suart_get_buffer(buffer);" gets the whole buffer. After the buffer has been read the used chars
      are removed from the suart_rx_buffer and their space is reclaimed.
*/
      suart_puts_P("\r\n1) ACTUAL BUFFER CONTENTS: ");
      suart_puts((unsigned char*)suart_rx_buffer);
      suart_puts_P("\r\n2) RETRIEVED BUFFER DATA: ");
      suart_get_buf(buffer, suart_string_received);
      suart_puts(buffer);
      suart_puts_P("\r\n");
      
      suart_puts_P("Type at least 5 chars\r\n");
      suart_set_mark=5;                             /* set the max wanted received chars */
      suart_gets();
      suart_get_buf(buffer, suart_rx_count);
      suart_puts(buffer);                           /* After the buffer has been read the chars are */
      suart_puts_P("\r\n");                         /* no longer in the buffer and more space is usable */

#elif  SUART_RECEIVE_STRINGS_ONLY == 1
#warning " SUART IN STRING ONLY MODE "
      suart_puts(string);
      suart_puts_P("\r\n");
      /* ENABLE STRING RECEIVING */
      suart_puts_P("Type something\r\n");
      suart_gets();
#if   SUART_WILL_USE_INTERRUPT == 1
      while(suart_string_received == 0);           /* NEEDED ONLY IN INTERRUPT MODE */
#endif
      suart_puts_P("sizeof(received_string)= ");
      suart_puti(suart_string_received, 0);
/*
      "suart_get_buffer(buffer);" gets the whole buffer. After the buffer has been read the used chars
      are removed from the suart_rx_buffer and their space is reclaimed.
*/
      suart_puts_P("\r\n1) ACTUAL BUFFER CONTENTS: ");
      suart_puts((unsigned char*)suart_rx_buffer);
      suart_puts_P("\r\n2) RETRIEVED BUFFER DATA: ");
      suart_get_buf(buffer, suart_string_received);
      suart_puts(buffer);
      suart_puts_P("\r\n");      

#endif

      suart_puts_e(eeprom_string);
      suart_puts_P("\r\nsuart_puti Testing...   ");
      suart_puti(1963,1);
      suart_putc('\r');
      suart_putc('\n');
      


      suart_puts_P("Press ENTER to repeat");
      while(suart_getc()!='\r');
      suart_putc('\r');
      suart_putc('\n');
      suart_puts_P("\r\n");
      
    }
        
}
/*########################################################################################################*/
/*                                     T   H   E      E   N   D                                           */
/*########################################################################################################*/


