/****************************************************************************
 Title  :   S file for the SOFTWARE UART MINIMUM FUNCTIONS  (softuart.s)
 Author:    Chris efstathiou hendrix@otenet.gr
 Date:      6/Jun/2002
 Software:  AVR-GCC with AVR-AS
 Target:    any AVR device
 Comments:  This software is FREE.

*****************************************************************************/
#ifndef _IO_REG_MACRO_MODE_
#define _IO_REG_MACRO_MODE_  1     /* In case you have the new assignment mode io headers */
#endif

#ifndef  _SFR_ASM_COMPAT
#define  _SFR_ASM_COMPAT     1     /* This is for GCC 3.2 */
#endif

#include <io-avr.h>
#include "suart_min.cfg"

/******CONFIGURATION BLOCK START  Change the below values to adapt settings******************/
 #if SREG > 0X3F

#define S_TX_DDR_REG  SUART_TX_DDR_REG - 0X20
#define S_TX_PORT     SUART_TX_PORT    - 0X20
#define S_RX_DDR_REG  SUART_RX_DDR_REG - 0X20
#define S_RX_PIN_REG  SUART_RX_PIN_REG - 0X20
#define S_INT_REG     SUART_INT_REG    - 0X20

#else

#define S_TX_DDR_REG  SUART_TX_DDR_REG
#define S_TX_PORT     SUART_TX_PORT
#define S_RX_DDR_REG  SUART_RX_DDR_REG
#define S_RX_PIN_REG  SUART_RX_PIN_REG
#define S_INT_REG     SUART_INT_REG

#endif
/****************************CONFIGURATION BLOCK END******************************************/ 


/* DO NOT CHANGE ANYTHING BELOW THIS LINE !!! */
#define          bitcnt  r30                     /* bit counter */
#define          temp    r31                     /* temporary storage register */
#define          Txbyte  r24                     /* r24,r25 are used for passing parameters */
#define          Rxbyte  r24                     /* r24,r25 are used for returning a value in gcc */

/*********************************************************************************************/
                 .global suart_putc
                 .func suart_putc

suart_putc:
#if SUART_TX_PORT_IS_IO == 1
                 sbi      S_TX_DDR_REG, TxD      ; set port Txd bit as output
#endif
                 ldi      bitcnt, 9+sb
                 com      Txbyte                 ; Invert everything
                 sec                             ; Start bit

putchar0:        brcc     putchar1               ; If carry set
                 cbi      S_TX_PORT, TxD         ; send a '0'
                 rjmp     putchar2               ; else

putchar1:        sbi      S_TX_PORT, TxD         ; send a '1'
                 nop

putchar2:
                 ldi      temp, SUART_DELAY2     ; 0,5 bit delay ((3*b)+9)*1/F_CPU
SUART_tx_delay1: dec      temp
                 brne     SUART_tx_delay1
suart_tx_nop:                                    ; add extra nop commands here

                 ldi      temp, SUART_DELAY3     ; another 0,5 bit delay making 1 bit in total
SUART_tx_delay2: dec      temp
                 brne     SUART_tx_delay2


                 lsr      Txbyte                 ; Get next bit
                 dec      bitcnt                 ; decrement bitcount
                 brne     putchar0               ; If not all bit sent (if not 0) send next

                 ret                             ; return from function

.endfunc
/***************************************************************************************************/
                 .global suart_getc
                 .func suart_getc

suart_getc:      
#if SUART_RX_PORT_IS_IO == 1                 
                 cbi     S_RX_DDR_REG, RxD  ; set port Rxd bit as input
#endif                 
                 ldi     bitcnt, 8               ; 8 data bit
suart_getchar0:  sbis    S_RX_PIN_REG, RxD       ; Wait for low-high-low transition
                 rjmp    suart_getchar0          ; or for a high-low transition
suart_getchar1:  sbic    S_RX_PIN_REG, RxD       ;
                 rjmp    suart_getchar1          ; Wait for start bit

                 ldi     temp, SUART_DELAY1      ; 0,5 bit -3c delay to be counted only once (start bit)
SUART_rx_delay1: dec     temp
                 brne    SUART_rx_delay1

getchar2:        ldi     temp, SUART_DELAY2       ; another 0,5 bit delay ((3*b)+9)*1/F_CPU
SUART_rx_delay2: dec     temp
                 brne    SUART_rx_delay2
suart_rx_nop:                                    ; add extra nop commands here

                 ldi     temp, SUART_DELAY3      ; another 0,5 bit delay thus making 1 bit in total
SUART_rx_delay3: dec     temp
                 brne    SUART_rx_delay3

                 clc                             ; clear carry
                 sbic    S_RX_PIN_REG, RxD       ; if RX pin high
                 sec                             ; set cary
                                                 ; else
                 ror     Rxbyte                  ; shift bit into Rxbyte
                 dec     bitcnt                  ; If bit is stop bit
                 breq    getchar3                ; return
                 rjmp    getchar2                ; else go get next bit

getchar3:        ret                             ; return from function


.endfunc
/***************************************************************************************************/
/*######################################################################################################*/
/*                                         T H E   E N D                                                */
/*######################################################################################################*/

